@@ -42,6 +42,10 @@
  *    large number of images will be displayed on the screen.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
@@ -68,10 +72,14 @@ static const char *tests[] = {
                               "boxa3_reg",
                               "boxa4_reg",
                               "bytea_reg",
+                              "ccbord_reg",
                               "ccthin1_reg",
                               "ccthin2_reg",
+                              "checkerboard_reg",
+                              "circle_reg",
                               "cmapquant_reg",
                               "colorcontent_reg",
+                              "colorfill_reg",
                               "coloring_reg",
                               "colorize_reg",
                               "colormask_reg",
@@ -84,6 +92,7 @@ static const char *tests[] = {
                               "conncomp_reg",
                               "conversion_reg",
                               "convolve_reg",
+                              "crop_reg",
                               "dewarp_reg",
                               "distance_reg",
                               "dither_reg",
@@ -110,6 +119,8 @@ static const char *tests[] = {
                               "graymorph2_reg",
                               "grayquant_reg",
                               "hardlight_reg",
+                              "hash_reg",
+                              "heap_reg",
                               "insert_reg",
                               "ioformats_reg",
                               "iomisc_reg",
@@ -125,6 +136,7 @@ static const char *tests[] = {
                               "locminmax_reg",
                               "logicops_reg",
                               "lowaccess_reg",
+                              "lowsat_reg",
                               "maze_reg",
                               "mtiff_reg",
                               "multitype_reg",
@@ -136,7 +148,10 @@ static const char *tests[] = {
                               "pageseg_reg",
                               "paint_reg",
                               "paintmask_reg",
+                              "pdfio1_reg",
+                              "pdfio2_reg",
                               "pdfseg_reg",
+                              "pixa1_reg",
                               "pixa2_reg",
                               "pixadisp_reg",
                               "pixcomp_reg",
@@ -157,6 +172,7 @@ static const char *tests[] = {
                               "rankhisto_reg",
                               "rasterop_reg",
                               "rasteropip_reg",
+                              "rectangle_reg",
                               "rotate1_reg",
                               "rotate2_reg",
                               "rotateorth_reg",
@@ -166,6 +182,7 @@ static const char *tests[] = {
                               "shear1_reg",
                               "shear2_reg",
                               "skew_reg",
+                              "smallpix_reg",
                               "speckle_reg",
                               "splitcomp_reg",
                               "string_reg",
@@ -175,6 +192,9 @@ static const char *tests[] = {
                               "translate_reg",
                               "warper_reg",
                               "watershed_reg",
+#if HAVE_LIBWEBP_ANIM
+                              "webpanimio_reg",
+#endif  /* HAVE_LIBWEBP_ANIM */
 #if HAVE_LIBWEBP
                               "webpio_reg",
 #endif  /* HAVE_LIBWEBP */
@@ -203,8 +223,8 @@ static char  mainName[] = "alltests_reg";
     setLeptDebugOK(1);  /* required for testing */
     l_getCurrentTime(&start, NULL);
     ntests = sizeof(tests) / sizeof(char *);
-    fprintf(stderr, "Running alltests_reg:\n"
-            "This currently tests %d of the 128 regression test\n"
+    lept_stderr("Running alltests_reg:\n"
+            "This currently tests %d regression test\n"
             "programs in the /prog directory.\n", ntests);
 
         /* Clear the output file if we're doing the set of reg tests */
@@ -237,7 +257,7 @@ static char  mainName[] = "alltests_reg";
                 nfail++;
             }
             else
-                fprintf(stderr, "%s", buf);
+                lept_stderr("%s", buf);
         }
     }
 
@@ -249,11 +269,11 @@ static char  mainName[] = "alltests_reg";
 #endif  /* !_WIN32 */
         lept_free(results_file);
         ret = system(command);
-        fprintf(stderr, "Success in %d of %d *_reg programs (output matches"
+        lept_stderr("Success in %d of %d *_reg programs (output matches"
                 " the \"golden\" files)\n", ntests - nfail, ntests);
     }
 
     l_getCurrentTime(&stop, NULL);
-    fprintf(stderr, "Time for all regression tests: %d sec\n", stop - start);
+    lept_stderr("Time for all regression tests: %d sec\n", stop - start);
     return 0;
 }
