@@ -67,6 +67,7 @@
  *           l_int32     pixcmapGetRangeValues()
  *
  *      Colormap conversion
+ *           PIXCMAP    *pixcmapGrayToFalseColor()
  *           PIXCMAP    *pixcmapGrayToColor()
  *           PIXCMAP    *pixcmapColorToGray()
  *           PIXCMAP    *pixcmapConvertTo4()
@@ -92,9 +93,21 @@
  *           l_int32     pixcmapContrastTRC()
  *           l_int32     pixcmapShiftIntensity()
  *           l_int32     pixcmapShiftByComponent()
+ *
+ *  Note:
+ *      (1) colormaps in leptonica have a maximum of 256 entries.
+ *      (2) nalloc, the allocated size of the palette array, is related
+ *          to the depth d of the pixels by:
+ *                 nalloc = 2^(d)
+ *
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
+#include <math.h>
 #include <string.h>
 #include "allheaders.h"
 
@@ -104,7 +117,7 @@
 /*!
  * \brief   pixcmapCreate()
  *
- * \param[in]    depth bpp, of pix
+ * \param[in]    depth    of pix, in bpp
  * \return  cmap, or NULL on error
  */
 PIXCMAP *
@@ -131,9 +144,9 @@ PIXCMAP    *cmap;
 /*!
  * \brief   pixcmapCreateRandom()
  *
- * \param[in]    depth bpp, of pix; 2, 4 or 8
- * \param[in]    hasblack 1 if the first color is black; 0 if no black
- * \param[in]    haswhite 1 if the last color is white; 0 if no white
+ * \param[in]    depth      of pix, in bpp: 2, 4 or 8
+ * \param[in]    hasblack   1 if the first color is black; 0 if no black
+ * \param[in]    haswhite   1 if the last color is white; 0 if no white
  * \return  cmap, or NULL on error
  *
  * <pre>
@@ -190,8 +203,8 @@ PIXCMAP  *cmap;
 /*!
  * \brief   pixcmapCreateLinear()
  *
- * \param[in]    d depth of pix for this colormap; 1, 2, 4 or 8
- * \param[in]    nlevels valid in range [2, 2^d]
+ * \param[in]    d          depth of pix for this colormap; 1, 2, 4 or 8
+ * \param[in]    nlevels    valid in range [2, 2^d]
  * \return  cmap, or NULL on error
  *
  * <pre>
@@ -231,7 +244,7 @@ PIXCMAP  *cmap;
  * \return  cmapd, or NULL on error
  */
 PIXCMAP *
-pixcmapCopy(PIXCMAP  *cmaps)
+pixcmapCopy(const PIXCMAP  *cmaps)
 {
 l_int32   nbytes, valid;
 PIXCMAP  *cmapd;
@@ -240,7 +253,7 @@ PIXCMAP  *cmapd;
 
     if (!cmaps)
         return (PIXCMAP *)ERROR_PTR("cmaps not defined", procName, NULL);
-    pixcmapIsValid(cmaps, &valid);
+    pixcmapIsValid(cmaps, NULL, &valid);
     if (!valid)
         return (PIXCMAP *)ERROR_PTR("invalid cmap", procName, NULL);
 
@@ -258,7 +271,7 @@ PIXCMAP  *cmapd;
 /*!
  * \brief   pixcmapDestroy()
  *
- * \param[in,out]   pcmap set to null
+ * \param[in,out]   pcmap    set to null on return
  * \return  void
  */
 void
@@ -279,21 +292,32 @@ PIXCMAP  *cmap;
     LEPT_FREE(cmap->array);
     LEPT_FREE(cmap);
     *pcmap = NULL;
-    return;
 }
 
 /*!
  * \brief   pixcmapIsValid()
  *
  * \param[in]    cmap
- * \param[out]   pvalid   return 1 if valid; 0 if not
+ * \param[in]    pix        optional; can be NULL
+ * \param[out]   pvalid     return 1 if valid; 0 if not
  * \return  0 if OK, 1 on error or if cmap is not valid
+ *
+ * <pre>
+ * Notes:
+ *      (1) If %pix is input, this will veify that pixel values cannot
+ *          overflow the colormap.  This is a relatively expensive operation
+ *          that may need to check all the pixel values.
+ *      (2) If %pix is input, there must be at least one color in the
+ *          colormap if it is to be valid with any pix, even if the
+ *          pixels are all 0.
+ * </pre>
  */
 l_ok
-pixcmapIsValid(PIXCMAP  *cmap,
-               l_int32  *pvalid)
+pixcmapIsValid(const PIXCMAP  *cmap,
+               PIX            *pix,
+               l_int32        *pvalid)
 {
-l_int32  d;
+l_int32  d, nalloc, maxindex;
 
     PROCNAME("pixcmapIsValid");
 
@@ -305,30 +329,57 @@ l_int32  d;
     if (!cmap->array)
         return ERROR_INT("cmap array not defined", procName, 1);
     d = cmap->depth;
-    if (d !=1 && d != 2 && d != 4 && d != 8) {
+    if (d != 1 && d != 2 && d != 4 && d != 8) {
         L_ERROR("invalid cmap depth: %d\n", procName, d);
         return 1;
     }
-    if (cmap->nalloc < 2 || cmap->nalloc > 256) {
-        L_ERROR("invalid cmap nalloc: %d\n", procName, cmap->nalloc);
+    nalloc = cmap->nalloc;
+    if (nalloc != (1 << d)) {
+        L_ERROR("invalid cmap nalloc = %d; d = %d\n", procName, nalloc, d);
         return 1;
     }
-    if (cmap->n < 0 || cmap->n > 256 || cmap->n > cmap->nalloc) {
-        L_ERROR("invalid cmap n: %d (nalloc = %d)\n", procName,
-                cmap->n, cmap->nalloc);
+    if (cmap->n < 0 || cmap->n > nalloc) {
+        L_ERROR("invalid cmap n: %d; nalloc = %d\n", procName, cmap->n, nalloc);
         return 1;
     }
+
+        /* To prevent indexing overflow into the cmap, the pix depth
+         * must not exceed the cmap depth.  Do not require depth equality,
+         * because some functions such as median cut quantizers do not. */
+    if (pix && (pixGetDepth(pix) > d)) {
+        L_ERROR("(pix depth = %d) > (cmap depth = %d)\n", procName,
+                pixGetDepth(pix), d);
+        return 1;
+    }
+    if (pix && cmap->n < 1) {
+        L_ERROR("cmap array is empty; invalid with any pix\n", procName);
+        return 1;
+    }
+
+        /* Where the colormap or the pix may have been corrupted, and
+         * in particular when reading or writing image files, it should
+         * be verified that the image pixel values do not exceed the
+         * max indexing into the colormap array. */
+    if (pix) {
+        pixGetMaxColorIndex(pix, &maxindex);
+        if (maxindex >= cmap->n) {
+            L_ERROR("(max index = %d) >= (num colors = %d)\n", procName,
+                    maxindex, cmap->n);
+            return 1;
+        }
+    }
+
     *pvalid = 1;
     return 0;
 }
-        
+
 
 /*!
  * \brief   pixcmapAddColor()
  *
  * \param[in]    cmap
- * \param[in]    rval, gval, bval colormap entry to be added; each number
- *                                is in range [0, ... 255]
+ * \param[in]    rval, gval, bval   colormap entry to be added; each number
+ *                                  is in range [0, ... 255]
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -366,8 +417,8 @@ RGBA_QUAD  *cta;
  * \brief   pixcmapAddRGBA()
  *
  * \param[in]    cmap
- * \param[in]    rval, gval, bval, aval colormap entry to be added;
- *                                      each number is in range [0, ... 255]
+ * \param[in]    rval, gval, bval, aval   colormap entry to be added;
+ *                                        each number is in range [0, ... 255]
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -405,9 +456,9 @@ RGBA_QUAD  *cta;
  * \brief   pixcmapAddNewColor()
  *
  * \param[in]    cmap
- * \param[in]    rval, gval, bval colormap entry to be added; each number
- *                                is in range [0, ... 255]
- * \param[out]   pindex index of color
+ * \param[in]    rval, gval, bval    colormap entry to be added; each number
+ *                                   is in range [0, ... 255]
+ * \param[out]   pindex              index of color
  * \return  0 if OK, 1 on error; 2 if unable to add color
  *
  * <pre>
@@ -455,9 +506,9 @@ pixcmapAddNewColor(PIXCMAP  *cmap,
  * \brief   pixcmapAddNearestColor()
  *
  * \param[in]    cmap
- * \param[in]    rval, gval, bval colormap entry to be added; each number
- *                                is in range [0, ... 255]
- * \param[out]   pindex index of color
+ * \param[in]    rval, gval, bval    colormap entry to be added; each number
+ *                                   is in range [0, ... 255]
+ * \param[out]   pindex              index of color
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -504,9 +555,9 @@ pixcmapAddNearestColor(PIXCMAP  *cmap,
  * \brief   pixcmapUsableColor()
  *
  * \param[in]    cmap
- * \param[in]    rval, gval, bval colormap entry to be added; each number
- *                                is in range [0, ... 255]
- * \param[out]   pusable 1 if usable; 0 if not
+ * \param[in]    rval, gval, bval   colormap entry to be added; each number
+ *                                  is in range [0, ... 255]
+ * \param[out]   pusable            1 if usable; 0 if not
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -549,8 +600,8 @@ l_int32  index;
  * \brief   pixcmapAddBlackOrWhite()
  *
  * \param[in]    cmap
- * \param[in]    color 0 for black, 1 for white
- * \param[out]   pindex [optional] index of color; can be null
+ * \param[in]    color    0 for black, 1 for white
+ * \param[out]   pindex   [optional] index of color; can be null
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -597,8 +648,8 @@ l_int32  index;
  * \brief   pixcmapSetBlackAndWhite()
  *
  * \param[in]    cmap
- * \param[in]    setblack 0 for no operation; 1 to set darkest color to black
- * \param[in]    setwhite 0 for no operation; 1 to set lightest color to white
+ * \param[in]    setblack   0 for no operation; 1 to set darkest color to black
+ * \param[in]    setwhite   0 for no operation; 1 to set lightest color to white
  * \return  0 if OK, 1 on error
  */
 l_ok
@@ -632,7 +683,7 @@ l_int32  index;
  * \return  count, or 0 on error
  */
 l_int32
-pixcmapGetCount(PIXCMAP  *cmap)
+pixcmapGetCount(const PIXCMAP  *cmap)
 {
     PROCNAME("pixcmapGetCount");
 
@@ -680,7 +731,7 @@ pixcmapGetDepth(PIXCMAP  *cmap)
  * \brief   pixcmapGetMinDepth()
  *
  * \param[in]    cmap
- * \param[out]   pmindepth minimum depth to support the colormap
+ * \param[out]   pmindepth    minimum depth to support the colormap
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -744,7 +795,7 @@ pixcmapClear(PIXCMAP  *cmap)
  *
  * \param[in]    cmap
  * \param[in]    index
- * \param[out]   prval, pgval, pbval each color value
+ * \param[out]   prval, pgval, pbval    each color value
  * \return  0 if OK, 1 if not accessible caller should check
  */
 l_ok
@@ -779,7 +830,7 @@ RGBA_QUAD  *cta;
  *
  * \param[in]    cmap
  * \param[in]    index
- * \param[out]   pval32 32-bit rgb color value
+ * \param[out]   pval32     32-bit rgb color value
  * \return  0 if OK, 1 if not accessible caller should check
  *
  * <pre>
@@ -812,7 +863,7 @@ l_int32  rval, gval, bval;
  *
  * \param[in]    cmap
  * \param[in]    index
- * \param[out]   prval, pgval, pbval, paval each color value
+ * \param[out]   prval, pgval, pbval, paval    each color value
  * \return  0 if OK, 1 if not accessible caller should check
  */
 l_ok
@@ -850,7 +901,7 @@ RGBA_QUAD  *cta;
  *
  * \param[in]    cmap
  * \param[in]    index
- * \param[out]   pval32 32-bit rgba color value
+ * \param[out]   pval32     32-bit rgba color value
  * \return  0 if OK, 1 if not accessible caller should check
  */
 l_ok
@@ -878,8 +929,8 @@ l_int32  rval, gval, bval, aval;
  *
  * \param[in]    cmap
  * \param[in]    index
- * \param[in]    rval, gval, bval colormap entry to be reset; each number
- *                                is in range [0, ... 255]
+ * \param[in]    rval, gval, bval    colormap entry to be reset; each number
+ *                                   is in range [0, ... 255]
  * \return  0 if OK, 1 if not accessible caller should check
  *
  * <pre>
@@ -919,7 +970,7 @@ RGBA_QUAD  *cta;
  *
  * \param[in]    cmap
  * \param[in]    index
- * \param[in]    aval in range [0, ... 255]
+ * \param[in]    aval     in range [0, ... 255]
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -954,9 +1005,9 @@ RGBA_QUAD  *cta;
  * \brief   pixcmapGetIndex()
  *
  * \param[in]    cmap
- * \param[in]    rval, gval, bval colormap colors to search for; each number
- *                                is in range [0, ... 255]
- * \param[out]   pindex found index
+ * \param[in]    rval, gval, bval   colormap colors to search for; each number
+ *                                  is in range [0, ... 255]
+ * \param[out]   pindex             value of index found
  * \return  0 if found, 1 if not found caller must check
  */
 l_int32
@@ -995,7 +1046,7 @@ RGBA_QUAD  *cta;
  * \brief   pixcmapHasColor()
  *
  * \param[in]    cmap
- * \param[out]   pcolor TRUE if cmap has color; FALSE otherwise
+ * \param[out]   pcolor    TRUE if cmap has color; FALSE otherwise
  * \return  0 if OK, 1 on error
  */
 l_ok
@@ -1034,7 +1085,7 @@ l_int32  *rmap, *gmap, *bmap;
  * \brief   pixcmapIsOpaque()
  *
  * \param[in]    cmap
- * \param[out]   popaque TRUE if fully opaque: all entries are 255
+ * \param[out]   popaque     TRUE if fully opaque: all entries are 255
  * \return  0 if OK, 1 on error
  */
 l_ok
@@ -1068,8 +1119,8 @@ RGBA_QUAD  *cta;
  * \brief   pixcmapIsBlackAndWhite()
  *
  * \param[in]    cmap
- * \param[out]   pblackwhite TRUE if the cmap has only two colors:
- *                           black (0,0,0) and white (255,255,255)
+ * \param[out]   pblackwhite   TRUE if the cmap has only two colors:
+ *                             black (0,0,0) and white (255,255,255)
  * \return  0 if OK, 1 on error
  */
 l_ok
@@ -1105,7 +1156,7 @@ RGBA_QUAD  *cta;
  * \brief   pixcmapCountGrayColors()
  *
  * \param[in]    cmap
- * \param[out]   pngray number of gray colors
+ * \param[out]   pngray     number of gray colors
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1149,9 +1200,9 @@ l_int32  *array;
  * \brief   pixcmapGetRankIntensity()
  *
  * \param[in]    cmap
- * \param[in]    rankval 0.0 for darkest, 1.0 for lightest color
- * \param[out]   pindex the index into the colormap that
- *                      corresponds to the rank intensity color
+ * \param[in]    rankval   0.0 for darkest, 1.0 for lightest color
+ * \param[out]   pindex    the index into the colormap that corresponds
+ *                         to the rank intensity color
  * \return  0 if OK, 1 on error
  */
 l_ok
@@ -1192,9 +1243,9 @@ NUMA    *na, *nasort;
  * \brief   pixcmapGetNearestIndex()
  *
  * \param[in]    cmap
- * \param[in]    rval, gval, bval colormap colors to search for; each number
- *                                is in range [0, ... 255]
- * \param[out]   pindex the index of the nearest color
+ * \param[in]    rval, gval, bval   colormap colors to search for; each number
+ *                                  is in range [0, ... 255]
+ * \param[out]   pindex             the index of the nearest color
  * \return  0 if OK, 1 on error caller must check
  *
  * <pre>
@@ -1251,8 +1302,8 @@ RGBA_QUAD  *cta;
  * \brief   pixcmapGetNearestGrayIndex()
  *
  * \param[in]    cmap
- * \param[in]    val gray value to search for; in range [0, ... 255]
- * \param[out]   pindex the index of the nearest color
+ * \param[in]    val       gray value to search for; in range [0, ... 255]
+ * \param[out]   pindex    the index of the nearest color
  * \return  0 if OK, 1 on error caller must check
  *
  * <pre>
@@ -1306,8 +1357,8 @@ RGBA_QUAD  *cta;
  *
  * \param[in]    cmap
  * \param[in]    index
- * \param[in]    rval, gval, bval target color
- * \param[out]   pdist the distance from the cmap entry to target
+ * \param[in]    rval,    gval, bval target color
+ * \param[out]   pdist    the distance from the cmap entry to target
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1357,12 +1408,12 @@ RGBA_QUAD  *cta;
  * \brief   pixcmapGetRangeValues()
  *
  * \param[in]    cmap
- * \param[in]    select L_SELECT_RED, L_SELECT_GREEN, L_SELECT_BLUE or
- *                      L_SELECT_AVERAGE
- * \param[out]   pminval [optional] minimum value of component
- * \param[out]   pmaxval [optional] maximum value of component
- * \param[out]   pminindex [optional] index of minimum value
- * \param[out]   pmaxindex [optional] index of maximum value
+ * \param[in]    select      L_SELECT_RED, L_SELECT_GREEN, L_SELECT_BLUE or
+ *                           L_SELECT_AVERAGE
+ * \param[out]   pminval     [optional] minimum value of component
+ * \param[out]   pmaxval     [optional] maximum value of component
+ * \param[out]   pminindex   [optional] index of minimum value
+ * \param[out]   pmaxindex   [optional] index of maximum value
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1453,6 +1504,68 @@ l_int32  i, n, imin, imax, minval, maxval, rval, gval, bval, aveval;
 /*-------------------------------------------------------------*
  *                       Colormap conversion                   *
  *-------------------------------------------------------------*/
+/*!
+ * \brief   pixcmapGrayToFalseColor()
+ *
+ * \param[in]    gamma   (factor) 0.0 or 1.0 for default; > 1.0 for brighter;
+ *                       2.0 is quite nice
+ * \return  cmap, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This creates a colormap that maps from gray to false colors.
+ *          The colormap is modeled after the Matlap "jet" configuration.
+ * </pre>
+ */
+PIXCMAP *
+pixcmapGrayToFalseColor(l_float32 gamma)
+{
+l_int32    i, rval, gval, bval;
+l_int32   *curve;
+l_float32  invgamma, x;
+PIXCMAP   *cmap;
+
+    if (gamma <= 0.0) gamma = 1.0;
+
+        /* Generate curve for transition part of color map */
+    curve = (l_int32 *)LEPT_CALLOC(64, sizeof(l_int32));
+    invgamma = 1. / gamma;
+    for (i = 0; i < 64; i++) {
+        x = (l_float32)i / 64.;
+        curve[i] = (l_int32)(255. * powf(x, invgamma) + 0.5);
+    }
+
+    cmap = pixcmapCreate(8);
+    for (i = 0; i < 256; i++) {
+        if (i < 32) {
+            rval = 0;
+            gval = 0;
+            bval = curve[i + 32];
+        } else if (i < 96) {   /* 32 - 95 */
+            rval = 0;
+            gval = curve[i - 32];
+            bval = 255;
+        } else if (i < 160) {  /* 96 - 159 */
+            rval = curve[i - 96];
+            gval = 255;
+            bval = curve[159 - i];
+        } else if (i < 224) {  /* 160 - 223 */
+            rval = 255;
+            gval = curve[223 - i];
+            bval = 0;
+        } else {  /* 224 - 255 */
+            rval = curve[287 - i];
+            gval = 0;
+            bval = 0;
+        }
+        pixcmapAddColor(cmap, rval, gval, bval);
+    }
+
+    LEPT_FREE(curve);
+    return cmap;
+}
+
+
 /*!
  * \brief   pixcmapGrayToColor()
  *
@@ -1490,7 +1603,7 @@ PIXCMAP  *cmap;
  * \brief   pixcmapColorToGray()
  *
  * \param[in]    cmaps
- * \param[in]    rwt, gwt, bwt  non-negative; these should add to 1.0
+ * \param[in]    rwt, gwt, bwt    non-negative; these should add to 1.0
  * \return  cmap gray, or NULL on error
  *
  * <pre>
@@ -1650,7 +1763,7 @@ PIXCMAP  *cmap;
 /*!
  * \brief   pixcmapReadStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp     file stream
  * \return  cmap, or NULL on error
  */
 PIXCMAP *
@@ -1674,7 +1787,8 @@ PIXCMAP  *cmap;
     ignore = fscanf(fp, "Color    R-val    G-val    B-val   Alpha\n");
     ignore = fscanf(fp, "----------------------------------------\n");
 
-    cmap = pixcmapCreate(depth);
+    if ((cmap = pixcmapCreate(depth)) == NULL)
+        return (PIXCMAP *)ERROR_PTR("cmap not made", procName, NULL);
     for (i = 0; i < ncolors; i++) {
         if (fscanf(fp, "%3d       %3d      %3d      %3d      %3d\n",
                         &index, &rval, &gval, &bval, &aval) != 5) {
@@ -1690,8 +1804,8 @@ PIXCMAP  *cmap;
 /*!
  * \brief   pixcmapReadMem()
  *
- * \param[in]    data  serialization of pixcmap; in ascii
- * \param[in]    size  of data in bytes; can use strlen to get it
+ * \param[in]    data     serialization of pixcmap; in ascii
+ * \param[in]    size     of data in bytes; can use strlen to get it
  * \return  cmap, or NULL on error
  */
 PIXCMAP *
@@ -1723,8 +1837,8 @@ PIXCMAP  *cmap;
  * \return  0 if OK, 1 on error
  */
 l_ok
-pixcmapWrite(const char  *filename,
-             PIXCMAP     *cmap)
+pixcmapWrite(const char     *filename,
+             const PIXCMAP  *cmap)
 {
 l_int32  ret;
 FILE    *fp;
@@ -1750,13 +1864,13 @@ FILE    *fp;
 /*!
  * \brief   pixcmapWriteStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp      file stream
    \param[in]    cmap
  * \return  0 if OK, 1 on error
  */
 l_ok
-pixcmapWriteStream(FILE     *fp,
-                   PIXCMAP  *cmap)
+pixcmapWriteStream(FILE           *fp,
+                   const PIXCMAP  *cmap)
 {
 l_int32  *rmap, *gmap, *bmap, *amap;
 l_int32   i;
@@ -1790,8 +1904,8 @@ l_int32   i;
 /*!
  * \brief   pixcmapWriteMem()
  *
- * \param[out]   pdata data of serialized pixcmap; ascii
- * \param[out]   psize size of returned data
+ * \param[out]   pdata     data of serialized pixcmap; ascii
+ * \param[out]   psize     size of returned data
  * \param[in]    cmap
  * \return  0 if OK, 1 on error
  *
@@ -1801,9 +1915,9 @@ l_int32   i;
  * </pre>
  */
 l_ok
-pixcmapWriteMem(l_uint8  **pdata,
-                size_t    *psize,
-                PIXCMAP   *cmap)
+pixcmapWriteMem(l_uint8        **pdata,
+                size_t         *psize,
+                const PIXCMAP  *cmap)
 {
 l_int32  ret;
 FILE    *fp;
@@ -1847,17 +1961,19 @@ FILE    *fp;
 /*!
  * \brief   pixcmapToArrays()
  *
- * \param[in]    cmap colormap
- * \param[out]   prmap, pgmap, pbmap  colormap arrays
- * \param[out]   pamap [optional] alpha array
+ * \param[in]    cmap     colormap
+ * \param[out]   prmap    array of red values
+ * \param[out]   pgmap    array of green values
+ * \param[out]   pbmap    array of blue values
+ * \param[out]   pamap    [optional] array of alpha (transparency) values
  * \return  0 if OK; 1 on error
  */
 l_ok
-pixcmapToArrays(PIXCMAP   *cmap,
-                l_int32  **prmap,
-                l_int32  **pgmap,
-                l_int32  **pbmap,
-                l_int32  **pamap)
+pixcmapToArrays(const PIXCMAP  *cmap,
+                l_int32       **prmap,
+                l_int32       **pgmap,
+                l_int32       **pbmap,
+                l_int32       **pamap)
 {
 l_int32    *rmap, *gmap, *bmap, *amap;
 l_int32     i, ncolors;
@@ -1873,10 +1989,9 @@ RGBA_QUAD  *cta;
         return ERROR_INT("cmap not defined", procName, 1);
 
     ncolors = pixcmapGetCount(cmap);
-    if (((rmap = (l_int32 *)LEPT_CALLOC(ncolors, sizeof(l_int32))) == NULL) ||
-        ((gmap = (l_int32 *)LEPT_CALLOC(ncolors, sizeof(l_int32))) == NULL) ||
-        ((bmap = (l_int32 *)LEPT_CALLOC(ncolors, sizeof(l_int32))) == NULL))
-            return ERROR_INT("calloc fail for *map", procName, 1);
+    rmap = (l_int32 *)LEPT_CALLOC(ncolors, sizeof(l_int32));
+    gmap = (l_int32 *)LEPT_CALLOC(ncolors, sizeof(l_int32));
+    bmap = (l_int32 *)LEPT_CALLOC(ncolors, sizeof(l_int32));
     *prmap = rmap;
     *pgmap = gmap;
     *pbmap = bmap;
@@ -1901,9 +2016,9 @@ RGBA_QUAD  *cta;
 /*!
  * \brief   pixcmapToRGBTable()
  *
- * \param[in]    cmap colormap
- * \param[out]   ptab table of rgba values for the colormap
- * \param[out]   pncolors [optional] size of table
+ * \param[in]    cmap       colormap
+ * \param[out]   ptab       table of rgba values for the colormap
+ * \param[out]   pncolors   [optional] size of table
  * \return  0 if OK; 1 on error
  */
 l_ok
@@ -1923,10 +2038,8 @@ l_uint32  *tab;
         return ERROR_INT("cmap not defined", procName, 1);
 
     ncolors = pixcmapGetCount(cmap);
-    if (pncolors)
-        *pncolors = ncolors;
-    if ((tab = (l_uint32 *)LEPT_CALLOC(ncolors, sizeof(l_uint32))) == NULL)
-        return ERROR_INT("tab not made", procName, 1);
+    if (pncolors) *pncolors = ncolors;
+    tab = (l_uint32 *)LEPT_CALLOC(ncolors, sizeof(l_uint32));
     *ptab = tab;
 
     for (i = 0; i < ncolors; i++) {
@@ -1940,10 +2053,10 @@ l_uint32  *tab;
 /*!
  * \brief   pixcmapSerializeToMemory()
  *
- * \param[in]    cmap colormap
- * \param[in]    cpc components/color: 3 for rgb, 4 for rgba
- * \param[out]   pncolors number of colors in table
- * \param[out]   pdata binary string, cpc bytes per color
+ * \param[in]    cmap       colormap
+ * \param[in]    cpc        components/color: 3 for rgb, 4 for rgba
+ * \param[out]   pncolors   number of colors in table
+ * \param[out]   pdata      binary string, cpc bytes per color
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -1975,9 +2088,7 @@ l_uint8  *data;
 
     ncolors = pixcmapGetCount(cmap);
     *pncolors = ncolors;
-    if ((data = (l_uint8 *)LEPT_CALLOC((size_t)cpc * ncolors, sizeof(l_uint8)))
-             == NULL)
-        return ERROR_INT("data not made", procName, 1);
+    data = (l_uint8 *)LEPT_CALLOC((size_t)cpc * ncolors, sizeof(l_uint8));
     *pdata = data;
 
     for (i = 0; i < ncolors; i++) {
@@ -1995,9 +2106,9 @@ l_uint8  *data;
 /*!
  * \brief   pixcmapDeserializeFromMemory()
  *
- * \param[in]    data binary string, 3 or 4 bytes per color
- * \param[in]    cpc components/color: 3 for rgb, 4 for rgba
- * \param[in]    ncolors
+ * \param[in]    data      binary string, 3 or 4 bytes per color
+ * \param[in]    cpc       components/color: 3 for rgb, 4 for rgba
+ * \param[in]    ncolors   > 0
  * \return  cmap, or NULL on error
  */
 PIXCMAP *
@@ -2014,7 +2125,7 @@ PIXCMAP  *cmap;
         return (PIXCMAP *)ERROR_PTR("data not defined", procName, NULL);
     if (cpc != 3 && cpc != 4)
         return (PIXCMAP *)ERROR_PTR("cpc not 3 or 4", procName, NULL);
-    if (ncolors == 0)
+    if (ncolors <= 0)
         return (PIXCMAP *)ERROR_PTR("no entries", procName, NULL);
     if (ncolors > 256)
         return (PIXCMAP *)ERROR_PTR("ncolors > 256", procName, NULL);
@@ -2046,8 +2157,8 @@ PIXCMAP  *cmap;
 /*!
  * \brief   pixcmapConvertToHex()
  *
- * \param[in]    data  binary serialized data
- * \param[in]    ncolors in colormap
+ * \param[in]    data       binary serialized data
+ * \param[in]    ncolors    in colormap
  * \return  hexdata bracketed, space-separated ascii hex string,
  *                       or NULL on error.
  *
@@ -2106,10 +2217,10 @@ char     buf[4];
 /*!
  * \brief   pixcmapGammaTRC()
  *
- * \param[in]    cmap colormap
- * \param[in]    gamma gamma correction; must be > 0.0
- * \param[in]    minval  input value that gives 0 for output; can be < 0
- * \param[in]    maxval  input value that gives 255 for output; can be > 255
+ * \param[in]    cmap      colormap
+ * \param[in]    gamma     gamma correction; must be > 0.0
+ * \param[in]    minval    input value that gives 0 for output; can be < 0
+ * \param[in]    maxval    input value that gives 255 for output; can be > 255
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -2162,9 +2273,9 @@ NUMA    *nag;
 /*!
  * \brief   pixcmapContrastTRC()
  *
- * \param[in]    cmap colormap
- * \param[in]    factor generally between 0.0 [no enhancement]
- *                      and 1.0, but can be larger than 1.0
+ * \param[in]    cmap     colormap
+ * \param[in]    factor   generally between 0.0 [no enhancement]
+ *                        and 1.0, but can be larger than 1.0
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -2210,8 +2321,8 @@ NUMA    *nac;
 /*!
  * \brief   pixcmapShiftIntensity()
  *
- * \param[in]    cmap colormap
- * \param[in]    fraction between -1.0 and +1.0
+ * \param[in]    cmap       colormap
+ * \param[in]    fraction   between -1.0 and +1.0
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -2261,9 +2372,9 @@ l_int32  i, ncolors, rval, gval, bval;
 /*!
  * \brief   pixcmapShiftByComponent()
  *
- * \param[in]    cmap colormap
- * \param[in]    srcval source color: 0xrrggbb00
- * \param[in]    dstval target color: 0xrrggbb00
+ * \param[in]    cmap     colormap
+ * \param[in]    srcval   source color: 0xrrggbb00
+ * \param[in]    dstval   target color: 0xrrggbb00
  * \return  0 if OK; 1 on error
  *
  * <pre>
