@@ -123,16 +123,21 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
-static const l_int32  INITIAL_PTR_ARRAYSIZE = 20;   /* must be > 0 */
+    /* Bounds on array sizes */
+static const size_t  MaxPtrArraySize = 100000;
+static const size_t  InitialPtrArraySize = 20;      /*!< n'importe quoi */
 
     /* Static functions */
 static l_int32 fpixaExtendArray(FPIXA *fpixa);
 static l_int32 fpixaExtendArrayToSize(FPIXA *fpixa, l_int32 size);
 
-
 /*--------------------------------------------------------------------*
  *                     FPix Create/copy/destroy                       *
  *--------------------------------------------------------------------*/
@@ -392,9 +397,7 @@ FPIX       *fpix;
             LEPT_FREE(data);
         LEPT_FREE(fpix);
     }
-
     *pfpix = NULL;
-    return;
 }
 
 
@@ -717,15 +720,13 @@ FPIXA  *fpixa;
 
     PROCNAME("fpixaCreate");
 
-    if (n <= 0)
-        n = INITIAL_PTR_ARRAYSIZE;
+    if (n <= 0 || n > MaxPtrArraySize)
+        n = InitialPtrArraySize;
 
-    if ((fpixa = (FPIXA *)LEPT_CALLOC(1, sizeof(FPIXA))) == NULL)
-        return (FPIXA *)ERROR_PTR("fpixa not made", procName, NULL);
+    fpixa = (FPIXA *)LEPT_CALLOC(1, sizeof(FPIXA));
     fpixa->n = 0;
     fpixa->nalloc = n;
     fpixa->refcount = 1;
-
     if ((fpixa->fpix = (FPIX **)LEPT_CALLOC(n, sizeof(FPIX *))) == NULL) {
         fpixaDestroy(&fpixa);
         return (FPIXA *)ERROR_PTR("fpixa ptrs not made", procName, NULL);
@@ -821,9 +822,7 @@ FPIXA   *fpixa;
         LEPT_FREE(fpixa->fpix);
         LEPT_FREE(fpixa);
     }
-
     *pfpixa = NULL;
-    return;
 }
 
 
@@ -883,6 +882,7 @@ FPIX    *fpixc;
  * <pre>
  * Notes:
  *      (1) Doubles the size of the fpixa ptr array.
+ *      (2) The max number of fpix ptrs is 100000.
  * </pre>
  */
 static l_int32
@@ -906,25 +906,35 @@ fpixaExtendArray(FPIXA  *fpixa)
  *
  * <pre>
  * Notes:
- *      (1) If necessary, reallocs new fpixa ptrs array to %size.
+ *      (1) If necessary, reallocs new fpix ptr array to %size.
+ *      (2) The max number of fpix ptrs is 100K.
  * </pre>
  */
 static l_int32
 fpixaExtendArrayToSize(FPIXA   *fpixa,
                        l_int32  size)
 {
+size_t  oldsize, newsize;
+
     PROCNAME("fpixaExtendArrayToSize");
 
     if (!fpixa)
         return ERROR_INT("fpixa not defined", procName, 1);
-
-    if (size > fpixa->nalloc) {
-        if ((fpixa->fpix = (FPIX **)reallocNew((void **)&fpixa->fpix,
-                                 sizeof(FPIX *) * fpixa->nalloc,
-                                 size * sizeof(FPIX *))) == NULL)
-            return ERROR_INT("new ptr array not returned", procName, 1);
-        fpixa->nalloc = size;
+    if (fpixa->nalloc > MaxPtrArraySize)  /* belt & suspenders */
+        return ERROR_INT("fpixa has too many ptrs", procName, 1);
+    if (size > MaxPtrArraySize)
+        return ERROR_INT("size > 100K ptrs; too large", procName, 1);
+    if (size <= fpixa->nalloc) {
+        L_INFO("size too small; no extension\n", procName);
+        return 0;
     }
+
+    oldsize = fpixa->nalloc * sizeof(FPIX *);
+    newsize = size * sizeof(FPIX *);
+    if ((fpixa->fpix = (FPIX **)reallocNew((void **)&fpixa->fpix,
+                                           oldsize, newsize)) == NULL)
+        return ERROR_INT("new ptr array not returned", procName, 1);
+    fpixa->nalloc = size;
     return 0;
 }
 
@@ -1386,9 +1396,7 @@ DPIX       *dpix;
             LEPT_FREE(data);
         LEPT_FREE(dpix);
     }
-
     *pdpix = NULL;
-    return;
 }
 
 
