@@ -50,10 +50,11 @@
  *
  *       With transcoding
  *          l_int32              l_generateCIData()
- *          static l_int32       pixGenerateCIData()
+ *          l_int32              pixGenerateCIData()
  *          L_COMP_DATA         *l_generateFlateData()
  *          static L_COMP_DATA  *pixGenerateFlateData()
  *          static L_COMP_DATA  *pixGenerateJpegData()
+ *          static L_COMP_DATA  *pixGenerateJp2kData()
  *          static L_COMP_DATA  *pixGenerateG4Data()
  *          L_COMP_DATA         *l_generateG4Data()
  *
@@ -90,6 +91,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include <math.h>
 #include "allheaders.h"
@@ -99,13 +104,14 @@
  /* --------------------------------------------*/
 
     /* Typical scan resolution in ppi (pixels/inch) */
-static const l_int32  DEFAULT_INPUT_RES = 300;
+static const l_int32  DefaultInputRes = 300;
 
     /* Static helpers */
 static L_COMP_DATA  *l_generateJp2kData(const char *fname);
 static L_COMP_DATA  *pixGenerateFlateData(PIX *pixs, l_int32 ascii85flag);
 static L_COMP_DATA  *pixGenerateJpegData(PIX *pixs, l_int32 ascii85flag,
                                          l_int32 quality);
+static L_COMP_DATA  *pixGenerateJp2kData(PIX *pixs, l_int32 quality);
 static L_COMP_DATA  *pixGenerateG4Data(PIX *pixs, l_int32 ascii85flag);
 
 static l_int32       l_generatePdf(l_uint8 **pdata, size_t *pnbytes,
@@ -152,27 +158,29 @@ static l_int32   var_WRITE_DATE_AND_VERSION = 1;
 /*!
  * \brief   pixConvertToPdfData()
  *
- * \param[in]      pix all depths; cmap OK
- * \param[in]      type L_G4_ENCODE, L_JPEG_ENCODE, L_FLATE_ENCODE
- * \param[in]      quality used for JPEG only; 0 for default (75)
- * \param[out]     pdata pdf array
- * \param[out]     pnbytes number of bytes in pdf array
- * \param[in]      x, y location of lower-left corner of image, in pixels,
- *                      relative to the PostScript origin (0,0) at
- *                      the lower-left corner of the page)
- * \param[in]      res override the resolution of the input image, in ppi;
- *                     use 0 to respect the resolution embedded in the input
- * \param[in]      title [optional] pdf title
- * \param[in,out]  plpd ptr to lpd, which is created on the first invocation
- *                      and returned until last image is processed
- * \param[in]      position in image sequence: L_FIRST_IMAGE, L_NEXT_IMAGE,
- *                          L_LAST_IMAGE
+ * \param[in]      pix       all depths; cmap OK
+ * \param[in]      type      L_G4_ENCODE, L_JPEG_ENCODE, L_FLATE_ENCODE,
+ *                           L_JP2K_ENCODE
+ * \param[in]      quality   for jpeg: 1-100; 0 for default (75)
+ *                           for jp2k: 27-45; 0 for default (34)
+ * \param[out]     pdata     pdf array
+ * \param[out]     pnbytes   number of bytes in pdf array
+ * \param[in]      x, y      location of lower-left corner of image, in pixels,
+ *                           relative to the PostScript origin (0,0) at
+ *                           the lower-left corner of the page)
+ * \param[in]      res       override the resolution of the input image, in ppi;
+ *                           use 0 to respect resolution embedded in the input
+ * \param[in]      title     [optional] pdf title; can be null
+ * \param[in,out]  plpd      ptr to lpd; created on the first invocation and
+ *                           returned until last image is processed
+ * \param[in]      position  in image sequence: L_FIRST_IMAGE, L_NEXT_IMAGE,
+ *                           L_LAST_IMAGE
  * \return  0 if OK, 1 on error
  *
  * <pre>
  * Notes:
  *      (1) If %res == 0 and the input resolution field is 0,
- *          this will use DEFAULT_INPUT_RES.
+ *          this will use DefaultInputRes.
  *      (2) This only writes %data if it is the last image to be
  *          written on the page.
  *      (3) See comments in convertToPdf().
@@ -206,6 +214,10 @@ L_PDF_DATA   *lpd = NULL;
     *pnbytes = 0;
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
+    if (type != L_JPEG_ENCODE && type != L_G4_ENCODE &&
+        type != L_FLATE_ENCODE && type != L_JP2K_ENCODE) {
+        selectDefaultPdfEncoding(pix, &type);
+    }
     if (plpd) {  /* part of multi-page invocation */
         if (position == L_FIRST_IMAGE)
             *plpd = NULL;
@@ -227,7 +239,7 @@ L_PDF_DATA   *lpd = NULL;
         if (pixres > 0)
             res = pixres;
         else
-            res = DEFAULT_INPUT_RES;
+            res = DefaultInputRes;
     }
     xpt = x * 72. / res;
     ypt = y * 72. / res;
@@ -272,10 +284,12 @@ L_PDF_DATA   *lpd = NULL;
 /*!
  * \brief   ptraConcatenatePdfToData()
  *
- * \param[in]    pa_data ptra array of pdf strings, each for a single-page pdf file
- * \param[in]    sa string array [optional] of pathnames for input pdf files
- * \param[out]   pdata concatenated pdf data in memory
- * \param[out]   pnbytes number of bytes in pdf data
+ * \param[in]    pa_data    ptra array of pdf strings, each for a
+ *                          single-page pdf file
+ * \param[in]    sa         [optional] string array of pathnames for
+ *                          input pdf files; can be null
+ * \param[out]   pdata      concatenated pdf data in memory
+ * \param[out]   pnbytes    number of bytes in pdf data
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -431,14 +445,14 @@ NUMAA    *naa_objs;  /* object mapping numbers to new values */
     l_byteaDestroy(&bad);
 
 #if  DEBUG_MULTIPAGE
-    fprintf(stderr, "******** object mapper **********");
+    lept_stderr("******** object mapper **********");
     numaaWriteStream(stderr, naa_objs);
 
-    fprintf(stderr, "******** Page object numbers ***********");
-    numaWriteStream(stderr, napage);
+    lept_stderr("******** Page object numbers ***********");
+    numaWriteStderr(napage);
 
-    fprintf(stderr, "******** Pages object ***********\n");
-    fprintf(stderr, "%s\n", str_pages);
+    lept_stderr("******** Pages object ***********\n");
+    lept_stderr("%s\n", str_pages);
 #endif  /* DEBUG_MULTIPAGE */
 
     numaDestroy(&napage);
@@ -457,7 +471,7 @@ NUMAA    *naa_objs;  /* object mapping numbers to new values */
 /*!
  * \brief   convertTiffMultipageToPdf()
  *
- * \param[in]    filein   (tiff)
+ * \param[in]    filein    (tiff)
  * \param[in]    fileout   (pdf)
  * \return  0 if OK, 1 on error
  *
@@ -497,10 +511,11 @@ FILE    *fp;
 /*!
  * \brief   l_generateCIDataForPdf()
  *
- * \param[in]    fname [optional]; can be null
- * \param[in]    pix [optional]; can be null
- * \param[in]    quality for jpeg if transcoded; 75 is standard
- * \param[out]   pcid compressed data
+ * \param[in]    fname      [optional] can be null
+ * \param[in]    pix        [optional] can be null
+ * \param[in]    quality    for jpeg if transcoded: 1-100; 0 for default (75)
+ *                          for jp2k if transcoded: 27-45; 0 for default (34)
+ * \param[out]   pcid       compressed data
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -563,7 +578,10 @@ PIX          *pixt;
             pixt = pixClone(pix);
         if (!pixt)
             return ERROR_INT("pixt not made", procName, 1);
-        selectDefaultPdfEncoding(pixt, &type);
+        if (selectDefaultPdfEncoding(pixt, &type)) {
+            pixDestroy(&pixt);
+            return 1;
+        }
         pixGenerateCIData(pixt, type, quality, 0, &cid);
         pixDestroy(&pixt);
     }
@@ -579,8 +597,8 @@ PIX          *pixt;
 /*!
  * \brief   l_generateFlateDataPdf()
  *
- * \param[in]    fname preferably png
- * \param[in]    pixs [optional]; can be null
+ * \param[in]    fname     preferably png
+ * \param[in]    pixs      [optional] can be null
  * \return  cid containing png data, or NULL on error
  *
  * <pre>
@@ -626,7 +644,8 @@ PIXCMAP      *cmap = NULL;
     bps = 0;  /* initialize to a nonsense value */
     if (format == IFF_PNG) {
         isPngInterlaced(fname, &interlaced);
-        readHeaderPng(fname, NULL, NULL, &bps, &spp, NULL);
+        if (readHeaderPng(fname, NULL, NULL, &bps, &spp, NULL))
+            return (L_COMP_DATA *)ERROR_PTR("bad png input", procName, NULL);
     }
 
         /* PDF is capable of inlining some types of PNG files, but not all
@@ -696,12 +715,12 @@ PIXCMAP      *cmap = NULL;
         n += pngcomp[i - 7] << 16;
         n += pngcomp[i - 6] << 8;
         n += pngcomp[i - 5] << 0;
-        if (i + n >= nbytespng) {
+        if (n >= nbytespng - i) {  /* "n + i" can overflow */
             LEPT_FREE(pngcomp);
             LEPT_FREE(datacomp);
             pixcmapDestroy(&cmap);
-            L_ERROR("invalid png: i = %d, n = %d, nbytes = %lu\n", procName,
-                    i, n, (unsigned long)nbytespng);
+            L_ERROR("invalid png: i = %d, n = %d, nbytes = %zu\n", procName,
+                    i, n, nbytespng);
             return NULL;
         }
 
@@ -778,8 +797,8 @@ PIXCMAP      *cmap = NULL;
 /*!
  * \brief   l_generateJpegData()
  *
- * \param[in]    fname of jpeg file
- * \param[in]    ascii85flag 0 for jpeg; 1 for ascii85-encoded jpeg
+ * \param[in]    fname           of jpeg file
+ * \param[in]    ascii85flag     0 for jpeg; 1 for ascii85-encoded jpeg
  * \return  cid containing jpeg data, or NULL on error
  *
  * <pre>
@@ -789,7 +808,8 @@ PIXCMAP      *cmap = NULL;
  *           ~ 1 for ascii85 (5 for 4) encoded binary data
  *               (not permitted in pdf)
  *      (2) Do not free the data.  l_generateJpegDataMem() will free
- *          the data if it does not use ascii encoding.
+ *          the data if the data is invalid, or if it does not use
+ *          ascii encoding.
  * </pre>
  */
 L_COMP_DATA *
@@ -816,9 +836,9 @@ size_t        nbytes;
 /*!
  * \brief   l_generateJpegDataMem()
  *
- * \param[in]    data     of jpeg file
- * \param[in]    nbytes
- * \param[in]    ascii85flag 0 for jpeg; 1 for ascii85-encoded jpeg
+ * \param[in]    data           of jpeg file
+ * \param[in]    nbytes         of jpeg file
+ * \param[in]    ascii85flag    0 for jpeg; 1 for ascii85-encoded jpeg
  * \return  cid containing jpeg data, or NULL on error
  *
  * <pre>
@@ -842,7 +862,10 @@ L_COMP_DATA  *cid;
         return (L_COMP_DATA *)ERROR_PTR("data not defined", procName, NULL);
 
         /* Read the metadata */
-    readHeaderMemJpeg(data, nbytes, &w, &h, &spp, NULL, NULL);
+    if (readHeaderMemJpeg(data, nbytes, &w, &h, &spp, NULL, NULL)) {
+        LEPT_FREE(data);
+        return (L_COMP_DATA *)ERROR_PTR("bad jpeg metadata", procName, NULL);
+    }
     bps = 8;
     readResolutionMemJpeg(data, nbytes, &xres, &yres);
 
@@ -877,7 +900,7 @@ L_COMP_DATA  *cid;
 /*!
  * \brief   l_generateJp2kData()
  *
- * \param[in]    fname of jp2k file
+ * \param[in]    fname     of jp2k file
  * \return  cid containing jp2k data, or NULL on error
  *
  * <pre>
@@ -888,15 +911,19 @@ L_COMP_DATA  *cid;
 static L_COMP_DATA *
 l_generateJp2kData(const char  *fname)
 {
-l_int32       w, h, bps, spp;
+l_int32       w, h, bps, spp, xres, yres;
 size_t        nbytes;
 L_COMP_DATA  *cid;
+FILE         *fp;
 
     PROCNAME("l_generateJp2kData");
 
     if (!fname)
         return (L_COMP_DATA *)ERROR_PTR("fname not defined", procName, NULL);
 
+    if (readHeaderJp2k(fname, &w, &h, &bps, &spp))
+        return (L_COMP_DATA *)ERROR_PTR("bad jp2k metadata", procName, NULL);
+
     if ((cid = (L_COMP_DATA *)LEPT_CALLOC(1, sizeof(L_COMP_DATA))) == NULL)
         return (L_COMP_DATA *)ERROR_PTR("cid not made", procName, NULL);
 
@@ -906,14 +933,18 @@ L_COMP_DATA  *cid;
         return (L_COMP_DATA *)ERROR_PTR("data not extracted", procName, NULL);
     }
 
-    readHeaderJp2k(fname, &w, &h, &bps, &spp);
+    xres = yres = 0;
+    if ((fp = fopenReadStream(fname)) != NULL) {
+        fgetJp2kResolution(fp, &xres, &yres);
+        fclose(fp);
+    }
     cid->type = L_JP2K_ENCODE;
     cid->nbytescomp = nbytes;
     cid->w = w;
     cid->h = h;
     cid->bps = bps;
     cid->spp = spp;
-    cid->res = 0;  /* don't know how to extract this */
+    cid->res = xres;
     return cid;
 }
 
@@ -922,10 +953,12 @@ L_COMP_DATA  *cid;
  * \brief   l_generateCIData()
  *
  * \param[in]    fname
- * \param[in]    type L_G4_ENCODE, L_JPEG_ENCODE, L_FLATE_ENCODE, L_JP2K_ENCODE
- * \param[in]    quality used for jpeg only; 0 for default (75)
- * \param[in]    ascii85 0 for binary; 1 for ascii85-encoded
- * \param[out]   pcid compressed data
+ * \param[in]    type       L_G4_ENCODE, L_JPEG_ENCODE, L_FLATE_ENCODE,
+ *                          L_JP2K_ENCODE
+ * \param[in]    quality    for jpeg if transcoded: 1-100; 0 for default (75)
+ *                          for jp2k if transcoded: 27-45; 0 for default (34)
+ * \param[in]    ascii85    0 for binary; 1 for ascii85-encoded
+ * \param[out]   pcid       compressed data
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -999,11 +1032,11 @@ PIX          *pix;
         } else {
             if ((pix = pixRead(fname)) == NULL)
                 return ERROR_INT("pix not returned", procName, 1);
-            cid = pixGenerateJpegData(pix, ascii85, quality);
+            cid = pixGenerateJp2kData(pix, quality);
             pixDestroy(&pix);
         }
         if (!cid)
-            return ERROR_INT("jpeg data not made", procName, 1);
+            return ERROR_INT("jp2k data not made", procName, 1);
     } else if (type == L_G4_ENCODE) {
         if ((cid = l_generateG4Data(fname, ascii85)) == NULL)
             return ERROR_INT("g4 data not made", procName, 1);
@@ -1022,11 +1055,13 @@ PIX          *pix;
 /*!
  * \brief   pixGenerateCIData()
  *
- * \param[in]    pixs 8 or 32 bpp, no colormap
- * \param[in]    type L_G4_ENCODE, L_JPEG_ENCODE, L_FLATE_ENCODE
- * \param[in]    quality used for jpeg only; 0 for default (75)
- * \param[in]    ascii85 0 for binary; 1 for ascii85-encoded
- * \param[out]   pcid compressed data
+ * \param[in]    pixs       8 or 32 bpp, no colormap
+ * \param[in]    type       L_G4_ENCODE, L_JPEG_ENCODE, L_FLATE_ENCODE or
+ *                          L_JP2K_ENCODE
+ * \param[in]    quality    for jpeg if transcoded: 1-100; 0 for default (75)
+ *                          for jp2k if transcoded: 27-45; 0 for default (34)
+ * \param[in]    ascii85    0 for binary; 1 for ascii85-encoded
+ * \param[out]   pcid       compressed data
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1054,18 +1089,42 @@ PIXCMAP  *cmap;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
     if (type != L_G4_ENCODE && type != L_JPEG_ENCODE &&
-        type != L_FLATE_ENCODE)
-        return ERROR_INT("invalid conversion type", procName, 1);
+        type != L_FLATE_ENCODE && type != L_JP2K_ENCODE) {
+        selectDefaultPdfEncoding(pixs, &type);
+    }
     if (ascii85 != 0 && ascii85 != 1)
         return ERROR_INT("invalid ascii85", procName, 1);
 
+        /* Conditionally modify the encoding type if libz is
+         * available and the requested library is missing. */
+#if defined(HAVE_LIBZ)
+# if !defined(HAVE_LIBJPEG)
+    if (type == L_JPEG_ENCODE) {
+        L_WARNING("no libjpeg; using flate encoding\n", procName);
+        type = L_FLATE_ENCODE;
+    }
+# endif /* !defined(HAVE_LIBJPEG) */
+# if !defined(HAVE_LIBJP2K)
+    if (type == L_JP2K_ENCODE) {
+        L_WARNING("no libjp2k; using flate encoding\n", procName);
+        type = L_FLATE_ENCODE;
+    }
+# endif /* !defined(HAVE_LIBJP2K) */
+# if !defined(HAVE_LIBTIFF)
+    if (type == L_G4_ENCODE) {
+        L_WARNING("no libtiff; using flate encoding\n", procName);
+        type = L_FLATE_ENCODE;
+    }
+# endif /* !defined(HAVE_LIBTIFF) */
+#endif /* defined(HAVE_LIBZ) */
+
         /* Sanity check on requested encoding */
     d = pixGetDepth(pixs);
     cmap = pixGetColormap(pixs);
     if (cmap && type != L_FLATE_ENCODE) {
         L_WARNING("pixs has cmap; using flate encoding\n", procName);
         type = L_FLATE_ENCODE;
-    } else if (d < 8 && type == L_JPEG_ENCODE) {
+    } else if (d < 8 && (type == L_JPEG_ENCODE || type == L_JP2K_ENCODE)) {
         L_WARNING("pixs has < 8 bpp; using flate encoding\n", procName);
         type = L_FLATE_ENCODE;
     } else if (d > 1 && type == L_G4_ENCODE) {
@@ -1076,16 +1135,16 @@ PIXCMAP  *cmap;
     if (type == L_JPEG_ENCODE) {
         if ((*pcid = pixGenerateJpegData(pixs, ascii85, quality)) == NULL)
             return ERROR_INT("jpeg data not made", procName, 1);
+    } else if (type == L_JP2K_ENCODE) {
+        if ((*pcid = pixGenerateJp2kData(pixs, quality)) == NULL)
+            return ERROR_INT("jp2k data not made", procName, 1);
     } else if (type == L_G4_ENCODE) {
         if ((*pcid = pixGenerateG4Data(pixs, ascii85)) == NULL)
             return ERROR_INT("g4 data not made", procName, 1);
-    } else if (type == L_FLATE_ENCODE) {
+    } else {  /* type == L_FLATE_ENCODE */
         if ((*pcid = pixGenerateFlateData(pixs, ascii85)) == NULL)
             return ERROR_INT("flate data not made", procName, 1);
-    } else {
-        return ERROR_INT("invalid conversion type", procName, 1);
     }
-
     return 0;
 }
 
@@ -1094,7 +1153,7 @@ PIXCMAP  *cmap;
  * \brief   l_generateFlateData()
  *
  * \param[in]    fname
- * \param[in]    ascii85flag 0 for gzipped; 1 for ascii85-encoded gzipped
+ * \param[in]    ascii85flag    0 for gzipped; 1 for ascii85-encoded gzipped
  * \return  cid flate compressed image data, or NULL on error
  *
  * <pre>
@@ -1133,15 +1192,18 @@ PIX          *pixs;
  * \brief   pixGenerateFlateData()
  *
  * \param[in]    pixs
- * \param[in]    ascii85flag 0 for gzipped; 1 for ascii85-encoded gzipped
+ * \param[in]    ascii85flag 0    for gzipped; 1 for ascii85-encoded gzipped
  * \return  cid flate compressed image data, or NULL on error
  *
- *      Notes:
- *          1) This should not be called with an RGBA pix (spp == 4; it
- *              will ignore the alpha channel.  Likewise, if called with a
- *              colormapped pix, the alpha component in the colormap will
- *              be ignored as it is for all leptonica operations
- *              on colormapped pix.
+ * <pre>
+ * Notes:
+ *     (1) If called with an RGBA pix (spp == 4), the alpha channel
+ *         will be removed, projecting a white backgrouond through
+ *         any transparency.
+ *     (2) If called with a colormapped pix, any transparency in the
+ *         alpha component in the colormap will be ignored, as it is
+ *         for all leptonica operations on colormapped pix.
+ * </pre>
  */
 static L_COMP_DATA *
 pixGenerateFlateData(PIX     *pixs,
@@ -1181,10 +1243,12 @@ PIXCMAP      *cmap;
         pixt = pixConvertTo8(pixs, cmapflag);
         cmap = pixGetColormap(pixt);
         d = pixGetDepth(pixt);
+    } else if (d == 32 && pixGetSpp(pixs) == 4) {  /* remove alpha */
+        pixt = pixAlphaBlendUniform(pixs, 0xffffff00);
     } else {
         pixt = pixClone(pixs);
     }
-    spp = (d == 32) ? 3 : 1;  /* ignores alpha */
+    spp = (d == 32) ? 3 : 1;
     bps = (d == 32) ? 8 : d;
 
         /* Extract and encode the colormap data as both ascii85 and hexascii  */
@@ -1251,9 +1315,9 @@ PIXCMAP      *cmap;
 /*!
  * \brief   pixGenerateJpegData()
  *
- * \param[in]    pixs 8 or 32 bpp, no colormap
- * \param[in]    ascii85flag 0 for jpeg; 1 for ascii85-encoded jpeg
- * \param[in]    quality 0 for default, which is 75
+ * \param[in]    pixs           8 or 32 bpp, no colormap
+ * \param[in]    ascii85flag    0 for jpeg; 1 for ascii85-encoded jpeg
+ * \param[in]    quality        0 for default, which is 75
  * \return  cid jpeg compressed data, or NULL on error
  *
  * <pre>
@@ -1284,10 +1348,63 @@ L_COMP_DATA  *cid;
 
         /* Compress to a temp jpeg file */
     fname = l_makeTempFilename();
-    pixWriteJpeg(fname, pixs, quality, 0);
+    if (pixWriteJpeg(fname, pixs, quality, 0)) {
+        LEPT_FREE(fname);
+        return NULL;
+    }
 
+        /* Generate the data */
     cid = l_generateJpegData(fname, ascii85flag);
-    lept_rmfile(fname);
+    if (lept_rmfile(fname) != 0)
+        L_ERROR("temp file %s was not deleted\n", procName, fname);
+    LEPT_FREE(fname);
+    return cid;
+}
+
+
+/*!
+ * \brief   pixGenerateJp2kData()
+ *
+ * \param[in]    pixs           8 or 32 bpp, no colormap
+ * \param[in]    quality        0 for default, which is 34
+ * \return  cid jp2k compressed data, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) The quality can be set between 27 (very poor) and 45
+ *          (nearly perfect).  Use 0 for default (34). Use 100 for lossless,
+ *          but this is very expensive and not recommended.
+ * </pre>
+ */
+static L_COMP_DATA *
+pixGenerateJp2kData(PIX     *pixs,
+                    l_int32  quality)
+{
+l_int32       d;
+char         *fname;
+L_COMP_DATA  *cid;
+
+    PROCNAME("pixGenerateJp2kData");
+
+    if (!pixs)
+        return (L_COMP_DATA *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (pixGetColormap(pixs))
+        return (L_COMP_DATA *)ERROR_PTR("pixs has colormap", procName, NULL);
+    d = pixGetDepth(pixs);
+    if (d != 8 && d != 32)
+        return (L_COMP_DATA *)ERROR_PTR("pixs not 8 or 32 bpp", procName, NULL);
+
+        /* Compress to a temp jp2k file */
+    fname = l_makeTempFilename();
+    if (pixWriteJp2k(fname, pixs, quality, 5, 0, 0)) {
+        LEPT_FREE(fname);
+        return NULL;
+    }
+
+        /* Generate the data */
+    cid = l_generateJp2kData(fname);
+    if (lept_rmfile(fname) != 0)
+        L_ERROR("temp file %s was not deleted\n", procName, fname);
     LEPT_FREE(fname);
     return cid;
 }
@@ -1296,8 +1413,8 @@ L_COMP_DATA  *cid;
 /*!
  * \brief   pixGenerateG4Data()
  *
- * \param[in]    pixs 1 bpp
- * \param[in]    ascii85flag 0 for gzipped; 1 for ascii85-encoded gzipped
+ * \param[in]    pixs           1 bpp
+ * \param[in]    ascii85flag    0 for gzipped; 1 for ascii85-encoded gzipped
  * \return  cid g4 compressed image data, or NULL on error
  *
  * <pre>
@@ -1311,7 +1428,7 @@ static L_COMP_DATA *
 pixGenerateG4Data(PIX     *pixs,
                   l_int32  ascii85flag)
 {
-char         *tname;
+char         *fname;
 L_COMP_DATA  *cid;
 
     PROCNAME("pixGenerateG4Data");
@@ -1322,12 +1439,16 @@ L_COMP_DATA  *cid;
         return (L_COMP_DATA *)ERROR_PTR("pixs not 1 bpp", procName, NULL);
 
         /* Compress to a temp tiff g4 file */
-    tname = l_makeTempFilename();
-    pixWrite(tname, pixs, IFF_TIFF_G4);
+    fname = l_makeTempFilename();
+    if (pixWrite(fname, pixs, IFF_TIFF_G4)) {
+        LEPT_FREE(fname);
+        return NULL;
+    }
 
-    cid = l_generateG4Data(tname, ascii85flag);
-    lept_rmfile(tname);
-    LEPT_FREE(tname);
+    cid = l_generateG4Data(fname, ascii85flag);
+    if (lept_rmfile(fname) != 0)
+        L_ERROR("temp file %s was not deleted\n", procName, fname);
+    LEPT_FREE(fname);
     return cid;
 }
 
@@ -1335,8 +1456,8 @@ L_COMP_DATA  *cid;
 /*!
  * \brief   l_generateG4Data()
  *
- * \param[in]    fname of g4 compressed file
- * \param[in]    ascii85flag 0 for g4 compressed; 1 for ascii85-encoded g4
+ * \param[in]    fname          of g4 compressed file
+ * \param[in]    ascii85flag    0 for g4 compressed; 1 for ascii85-encoded g4
  * \return  cid g4 compressed image data, or NULL on error
  *
  * <pre>
@@ -1412,10 +1533,10 @@ FILE         *fp;
 /*!
  * \brief   cidConvertToPdfData()
  *
- * \param[in]    cid compressed image data -- of jp2k image
- * \param[in]    title [optional] pdf title; can be NULL
- * \param[out]   pdata output pdf data for image
- * \param[out]   pnbytes size of output pdf data
+ * \param[in]    cid       compressed image data
+ * \param[in]    title     [optional] pdf title; can be NULL
+ * \param[out]   pdata     output pdf data for image
+ * \param[out]   pnbytes   size of output pdf data
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1446,7 +1567,7 @@ L_PDF_DATA  *lpd = NULL;
         /* Get media box parameters, in pts */
     res = cid->res;
     if (res <= 0)
-        res = DEFAULT_INPUT_RES;
+        res = DefaultInputRes;
     wpt = cid->w * 72. / res;
     hpt = cid->h * 72. / res;
 
@@ -1470,7 +1591,7 @@ L_PDF_DATA  *lpd = NULL;
 /*!
  * \brief   l_CIDataDestroy()
  *
- * \param[in,out]   pcid will be set to null before returning
+ * \param[in,out]   pcid     will be set to null before returning
  * \return  void
  */
 void
@@ -1493,7 +1614,6 @@ L_COMP_DATA  *cid;
     if (cid->cmapdatahex) LEPT_FREE(cid->cmapdatahex);
     LEPT_FREE(cid);
     *pcid = NULL;
-    return;
 }
 
 
@@ -1503,9 +1623,9 @@ L_COMP_DATA  *cid;
 /*!
  * \brief   l_generatePdf()
  *
- * \param[out]   pdata pdf array
- * \param[out]   pnbytes number of bytes in pdf array
- * \param[in]    lpd all the required input image data
+ * \param[out]   pdata     pdf array
+ * \param[out]   pnbytes   number of bytes in pdf array
+ * \param[in]    lpd       all the required input image data
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1613,14 +1733,13 @@ SARRAY  *sa;
     lpd->poststream = stringNew("\n"
                                 "endstream\n"
                                 "endobj\n");
-    return;
 }
 
 
 /*!
  * \brief   generateEscapeString()
  *
- * \param[in]   str   input string
+ * \param[in]   str      input string
  * \return   hex escape string, or null on error
  *
  * <pre>
@@ -1691,8 +1810,6 @@ l_float32  xpt, ypt, wpt, hpt, maxx, maxy;
         ptaGetPt(lpd->wh, i, &wpt, &hpt);
         ptaSetPt(lpd->xy, i, xpt, maxy - ypt - hpt);
     }
-
-    return;
 }
 
 
@@ -1891,7 +2008,7 @@ SARRAY       *sa;
         snprintf(buf, sizeof(buf),
                  "%d 0 obj\n"
                  "<<\n"
-                 "/Length %lu\n"
+                 "/Length %zu\n"
                  "/Subtype /Image\n"
                  "%s\n"  /* colorspace */
                  "/Width %d\n"
@@ -1901,7 +2018,7 @@ SARRAY       *sa;
                  "%s"   /* decode parms; can be empty */
                  ">>\n"
                  "stream\n",
-                 6 + i, (unsigned long)cid->nbytescomp, cstr,
+                 6 + i, cid->nbytescomp, cstr,
                  cid->w, cid->h, bstr, fstr, pstr);
         xstr = stringNew(buf);
         sarrayAddString(sa, xstr, L_INSERT);
@@ -2044,9 +2161,9 @@ SARRAY  *sa;
 /*!
  * \brief   generateOutputDataPdf()
  *
- * \param[out]   pdata pdf data array
- * \param[out]   pnbytes size of pdf data array
- * \param[in]    lpd input data used to make pdf
+ * \param[out]   pdata      pdf data array
+ * \param[out]   pnbytes    size of pdf data array
+ * \param[in]    lpd        input data used to make pdf
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -2126,8 +2243,8 @@ L_COMP_DATA  *cid;
 /*!
  * \brief   parseTrailerPdf()
  *
- * \param[in]    bas lba of a pdf file
- * \param[out]   pda byte locations of the beginning of each object
+ * \param[in]    bas     lba of a pdf file
+ * \param[out]   pda     byte locations of the beginning of each object
  * \return  0 if OK, 1 on error
  */
 static l_int32
@@ -2184,11 +2301,11 @@ SARRAY   *sa;
     l_dnaAddNumber(da, xrefloc);
 
 #if  DEBUG_MULTIPAGE
-    fprintf(stderr, "************** Trailer string ************\n");
-    fprintf(stderr, "xrefloc = %d", xrefloc);
+    lept_stderr("************** Trailer string ************\n");
+    lept_stderr("xrefloc = %d", xrefloc);
     sarrayWriteStream(stderr, sa);
 
-    fprintf(stderr, "************** Object locations ************");
+    lept_stderr("************** Object locations ************");
     l_dnaWriteStream(stderr, da);
 #endif  /* DEBUG_MULTIPAGE */
     sarrayDestroy(&sa);
@@ -2269,17 +2386,19 @@ SARRAY  *sa;
 /*!
  * \brief   substituteObjectNumbers()
  *
- *  Input:  bas (lba of a pdf object)
- *          na_objs (object number mapping array)
- *  Return: bad (lba of rewritten pdf for the object)
+ * \param[in]   bas        lba of a pdf object
+ * \param[in]   na_objs    object number mapping array
+ * \return    bad   lba of rewritten pdf for the object
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Interpret the first set of bytes as the object number,
  *          map to the new number, and write it out.
  *      (2) Find all occurrences of this 4-byte sequence: " 0 R"
  *      (3) Find the location and value of the integer preceding this,
  *          and map it to the new value.
  *      (4) Rewrite the object with new object numbers.
+ * </pre>
  */
 static L_BYTEA *
 substituteObjectNumbers(L_BYTEA  *bas,
@@ -2399,7 +2518,6 @@ L_PDF_DATA   *lpd;
     if (lpd->objloc) l_dnaDestroy(&lpd->objloc);
     LEPT_FREE(lpd);
     *plpd = NULL;
-    return;
 }
 
 
@@ -2424,8 +2542,8 @@ pdfdataGetCid(L_PDF_DATA  *lpd,
 /*!
  * \brief   l_pdfSetG4ImageMask()
  *
- * \param[in]    flag 1 for writing g4 data as fg only through a mask;
- *                    0 for writing fg and bg
+ * \param[in]    flag    1 for writing g4 data as fg only through a mask;
+ *                       0 for writing fg and bg
  * \return  void
  *
  * <pre>
@@ -2445,8 +2563,8 @@ l_pdfSetG4ImageMask(l_int32  flag)
 /*!
  * \brief   l_pdfSetDateAndVersion()
  *
- * \param[in]    flag 1 for writing date/time and leptonica version;
- *                    0 for omitting this from the metadata
+ * \param[in]    flag    1 for writing date/time and leptonica version;
+ *                       0 for omitting this from the metadata
  * \return  void
  *
  * <pre>
